#include "ggap/prefs.h"
#include "ggap/app.h"
#include <moo/macros.h>
#include <QtGui>

using namespace ggap;

namespace {

struct PrefsKey {
    QString name;
    QVariant dflt;

    PrefsKey(const QString &name, const QVariant &dflt = QVariant()) :
        name(name),
        dflt(dflt)
    {
    }
};

PrefsKey prefsKey(Prefs::BoolKey key)
{
    switch (key)
    {
        case Prefs::SaveWorkspace:
            return PrefsKey("gap/save-workspace", true);
        case Prefs::UseDefaultGap:
            return PrefsKey("gap/use-default", true);
        case Prefs::EnableTerminalMode:
            return PrefsKey("gap/enable-terminal-mode", false);
    }

    m_return_val_if_reached(PrefsKey(QString()));
}

PrefsKey prefsKey(Prefs::StringKey key)
{
    switch (key)
    {
        case Prefs::GapExe:
            return PrefsKey("gap/exe");
        case Prefs::GapRootDir:
            return PrefsKey("gap/rootdir");
        case Prefs::GapArgs:
            return PrefsKey("gap/args");
        case Prefs::HelpHomePage:
            return PrefsKey("help/home");
        case Prefs::DefaultFileFormat:
            return PrefsKey("gap-file/format", "workspace");
    }

    m_return_val_if_reached(PrefsKey(QString()));
}

PrefsKey prefsKey(Prefs::StringListKey key)
{
    switch (key)
    {
        case Prefs::HelpBookmarks:
            return PrefsKey("help/bookmarks");
        case Prefs::ExtraGapRoots:
            return PrefsKey("gap/extraroots");
        case Prefs::WatchGapFiles:
            return PrefsKey("gap/watchfiles");
    }

    m_return_val_if_reached(PrefsKey(QString()));
}

#ifdef Q_OS_MAC
#define DEFAULT_WORKSHEET_FONT "Monaco, 11"
#elif defined(Q_OS_WIN32)
#define DEFAULT_WORKSHEET_FONT "Courier New"
#else
#define DEFAULT_WORKSHEET_FONT "Monospace"
#endif

PrefsKey prefsKey(Prefs::FontKey key)
{
    switch (key)
    {
        case Prefs::WorksheetFont:
            return PrefsKey("worksheet/font", DEFAULT_WORKSHEET_FONT);
        case Prefs::HelpFont:
            return PrefsKey("help/font", QApplication::font());
    }

    m_return_val_if_reached(PrefsKey(QString()));
}

PrefsKey prefsKey(Prefs::IntKey key)
{
    switch (key)
    {
        case Prefs::CompressionLevel:
            return PrefsKey("gap-file/compression-level", -1);
    }

    m_return_val_if_reached(PrefsKey(QString()));
}

PrefsKey prefsKey(int key)
{
    if (0x100 < key && key < 0x110)
        return prefsKey(Prefs::BoolKey(key));
    if (0x110 < key && key < 0x120)
        return prefsKey(Prefs::StringKey(key));
    if (0x120 < key && key < 0x130)
        return prefsKey(Prefs::StringListKey(key));
    if (0x130 < key && key < 0x140)
        return prefsKey(Prefs::FontKey(key));
    if (0x140 < key && key < 0x150)
        return prefsKey(Prefs::IntKey(key));
    m_return_val_if_reached(PrefsKey(QString()));
}

}

QVariant ggap::impl::prefsValue(moo::SettingsType type, int key)
{
    PrefsKey pk = prefsKey(key);
    return moo::Prefs(type).value(pk.name, pk.dflt);
}

void ggap::impl::setPrefsValue(moo::SettingsType type, int key, const QVariant &value)
{
    PrefsKey pk = prefsKey(key);
    moo::Prefs(type).setValue(pk.name, value, pk.dflt);
}
