#include "moo/prefs.h"
#include "moo/app.h"
#include "moo/macros.h"
#include <QtGui>

using namespace moo;

QVariant Prefs::value(const QString &key, const QVariant &defaultValue) const
{
    return mooApp->settings(type)->value(key, defaultValue);
}

inline static bool equal_strings(const QString &s1, const QString &s2)
{
    if (s1.isEmpty())
        return s2.isEmpty();
    else if (s2.isEmpty())
        return false;
    else
        return s1 == s2;
}

inline static bool equal_fonts(const QString &fs1, const QString &fs2)
{
    QFont f1, f2;
    f1.fromString(fs1);
    f2.fromString(fs2);
    return f1 == f2;
}

inline static bool equal_values(const QVariant &v1, const QVariant &v2, bool *to_string = 0)
{
    if (to_string)
        *to_string = false;

    switch (v1.type())
    {
        case QVariant::String:
            return equal_strings(v1.toString(), v2.toString());

        case QVariant::ByteArray:
            return v1.toByteArray() == v2.toByteArray();

        case QVariant::Font:
            if (to_string)
                *to_string = true;
            return equal_fonts(v1.toString(), v2.toString());

        default:
            return v1 == v2;
    }
}

void Prefs::setValue(const QString &key, const QVariant &new_value, const QVariant &dflt)
{
    QVariant old_value = value(key, dflt);
    bool to_string = false;

    if (equal_values(new_value, old_value, &to_string))
        return;

    if (equal_values(new_value, dflt))
        mooApp->settings(type)->remove(key);
    else if (to_string)
        mooApp->settings(type)->setValue(key, new_value.toString());
    else
        mooApp->settings(type)->setValue(key, new_value);
}

void Prefs::unsetValue(const QString &key)
{
    mooApp->settings(type)->remove(key);
}

QByteArray moo::prefsValue(Prefs::ByteArrayKey key, const QByteArray &dflt)
{
    switch (key)
    {
        case Prefs::PageSetup:
            return prefsValue("print/page-setup", dflt).toByteArray();
    }

    m_return_val_if_reached(dflt);
}

void moo::setPrefsValue(Prefs::ByteArrayKey key, const QByteArray &value)
{
    switch (key)
    {
        case Prefs::PageSetup:
            setPrefsValue("print/page-setup", value);
            return;
    }

    m_return_if_reached();
}
